package cn.iocoder.yudao.module.customer.dal.mysql.customer;

import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.mapper.BaseMapperX;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.customer.dal.dataobject.customer.follow.CustomerFollowDO;
import cn.iocoder.yudao.module.customer.dto.follow.CustomerFollowPageDTO;
import cn.iocoder.yudao.module.customer.vo.customer.follow.CustomerFollowExportReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.follow.CustomerFollowPageReqVO;
import cn.iocoder.yudao.module.customer.vo.customer.follow.CustomerFollowRespVO;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.commons.lang3.StringUtils;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.ResultType;
import org.apache.ibatis.annotations.Select;

import java.util.Arrays;
import java.util.List;

/**
 * 客户投诉跟进 Mapper
 *
 * @author yanghao
 */
@Mapper
public interface CustomerFollowMapper extends BaseMapperX<CustomerFollowDO> {

    default PageResult<CustomerFollowDO> selectPage(CustomerFollowPageReqVO reqVO) {
        return selectPage(reqVO, new LambdaQueryWrapperX<CustomerFollowDO>()
                .eqIfPresent(CustomerFollowDO::getCustomerId, reqVO.getCustomerId())
                .eqIfPresent(CustomerFollowDO::getBizId, reqVO.getBizId())
                .likeIfPresent(CustomerFollowDO::getBizNo, reqVO.getBizNo())
                .eqIfPresent(CustomerFollowDO::getFollowUserId, reqVO.getFollowUserId())
                .inIfPresent(CustomerFollowDO::getFollowType, StringUtils.isNotBlank(reqVO.getFollowType()) ? Arrays.asList(reqVO.getFollowType().split(StrUtil.COMMA)) : null)
                .betweenIfPresent(CustomerFollowDO::getCreateTime, reqVO.getBeginCreateTime(), reqVO.getEndCreateTime())
                .orderByDesc(CustomerFollowDO::getId));
    }

    default List<CustomerFollowDO> selectList(CustomerFollowExportReqVO reqVO) {
        return selectList(new LambdaQueryWrapperX<CustomerFollowDO>()
                .eqIfPresent(CustomerFollowDO::getBizId, reqVO.getBizId())
                .eqIfPresent(CustomerFollowDO::getBizNo, reqVO.getBizNo())
                .betweenIfPresent(CustomerFollowDO::getCreateTime, reqVO.getBeginCreateTime(), reqVO.getEndCreateTime())
                .orderByDesc(CustomerFollowDO::getId));
    }

    @ResultType(CustomerFollowDO.class)
    @Select({
            "<script>",
            "select ",
            "f.*, ",
            "u.name as follow_user_name",
            "from ecw_customer_follow f ",
            "left join system_user u on u.id = o.follow_user_id ",
            "where eo.deleted = 0",
            "AND f.`follow_type` = 4",
            "AND f.`biz_id` = #{offerId}",
            "order by f.id desc",
            "</script>"
    })
    List<CustomerFollowDO> offerFollowList(@Param("offerId") Long offerId);

    @ResultType(CustomerFollowDO.class)
    @Select({
            "<script>",
            "select ",
            "f.*, ",
            "u.nickname as follow_user_name ",
            "from ecw_customer_follow f ",
            "left join system_user u on u.id = f.follow_user_Id ",
            "where f.deleted = 0",
            "<when test = 'types != null'>",
            "AND f.`follow_type` in ",
            "<foreach item='type' index='index' collection='types' open='(' separator=',' close=')'>#{type}</foreach> ",
            "</when>",
            "<when test = 'query.bizId != null'>",
            "AND f.`biz_id` = #{query.bizId}",
            "</when>",
            "<when test = 'query.bizNo != null and query.bizNo != \"\" '>",
            "AND f.`biz_no` like concat('%',concat(#{query.bizNo},'%'))",
            "</when>",
            "<when test = 'query.beginCreateTime != null and query.endCreateTime != null '>",
            "AND f.`follow_time` between #{query.beginCreateTime} and #{query.endCreateTime}",
            "</when>",
            "<when test = 'query.customerId != null'>",
            "AND f.`customer_id` = #{query.customerId}",
            "</when>",
            "<when test = 'query.bizId != null'>",
            "AND f.`biz_id` = #{query.bizId}",
            "</when>",
            "</script>"
    })
    IPage<CustomerFollowDO> customerFollowPage(@Param("mpPage") IPage<CustomerFollowDO> mpPage, @Param("query") CustomerFollowPageReqVO query, @Param("types") List<String> types);


    @ResultType(CustomerFollowRespVO.class)
    @Select({
            "<script>",
            "select ",
            "f.*,c.number, ",
            "t3.phone_new phone,",
            "u1.nickname as creatorName , ",
            "u2.nickname as updaterName , ",
            "u.nickname as follow_user_name ",
            "from ecw_customer_follow f ",
            "left join ecw_customer c on c.id = f.customer_id ",
            "left join system_user u on u.id = f.follow_user_Id ",
            "left join system_user u1 on u1.id = f.creator ",
            "left join system_user u2 on u2.id = f.updater ",

            "LEFT JOIN ecw_customer_contacts t3 ON c.id = t3.customer_id AND t3.is_default = 1 and t3.deleted=0" ,
            "where f.deleted = 0",
            "<when test = 'types != null'>",
            "AND f.`follow_type` in ",
            "<foreach item='type' index='index' collection='types' open='(' separator=',' close=')'>#{type}</foreach> ",
            "</when>",
            "<when test = 'query.followNo != null and query.followNo != \"\" '>",
            "AND f.`follow_no` like concat('%',concat(#{query.followNo},'%'))",
            "</when>",
            "<when test = 'query.contactName != null and query.contactName != \"\" '>",
            "AND f.`contact_name` like concat('%',concat(#{query.contactName},'%'))",
            "</when>",

            "<when test = 'query.beginCreateTime != null and query.endCreateTime != null '>",
            "AND f.`create_time` between #{query.beginCreateTime} and #{query.endCreateTime}",
            "</when>",
            "<when test = 'query.beginFollowTime != null and query.endFollowTime != null '>",
            "AND f.`follow_time` between #{query.beginFollowTime} and #{query.endFollowTime}",
            "</when>",
            "<when test = 'query.customerId != null'>",
            "AND f.`customer_id` = #{query.customerId}",
            "</when>",
            "<when test = 'query.bizId != null'>",
            "AND f.`biz_id` = #{query.bizId}",
            "</when>",
            "<when test = 'query.followUserId != null'>",
            "AND f.`follow_user_id` = #{query.followUserId}",
            "</when>",
            "group by f.id " ,
            " limit #{start}, #{size} " ,
            "</script>"
    })
    List<CustomerFollowRespVO> getFollowPage(@Param("start") int start, @Param("size") int size,  @Param("query") CustomerFollowPageDTO query, @Param("types") List<String> types);

    @Select({
            "<script>",
            "select count(*) ",

            "from ecw_customer_follow f ",
            "left join ecw_customer c on c.id = f.customer_id ",
            "left join system_user u on u.id = f.follow_user_Id ",
            "where f.deleted = 0",
            "<when test = 'types != null'>",
            "AND f.`follow_type` in ",
            "<foreach item='type' index='index' collection='types' open='(' separator=',' close=')'>#{type}</foreach> ",
            "</when>",
            "<when test = 'query.followNo != null and query.followNo != \"\" '>",
            "AND f.`follow_no` like concat('%',concat(#{query.followNo},'%'))",
            "</when>",
            "<when test = 'query.contactName != null and query.contactName != \"\" '>",
            "AND f.`contact_name` like concat('%',concat(#{query.contactName},'%'))",
            "</when>",

            "<when test = 'query.beginCreateTime != null and query.endCreateTime != null '>",
            "AND f.`create_time` between #{query.beginCreateTime} and #{query.endCreateTime}",
            "</when>",
            "<when test = 'query.beginFollowTime != null and query.endFollowTime != null '>",
            "AND f.`follow_time` between #{query.beginFollowTime} and #{query.endFollowTime}",
            "</when>",
            "<when test = 'query.customerId != null'>",
            "AND f.`customer_id` = #{query.customerId}",
            "</when>",
            "<when test = 'query.bizId != null'>",
            "AND f.`biz_id` = #{query.bizId}",
            "</when>",
            "<when test = 'query.followUserId != null'>",
            "AND f.`follow_user_id` = #{query.followUserId}",
            "</when>",
            "</script>"
    })
    long getFollowPageCount( @Param("query") CustomerFollowPageDTO query ,  @Param("types") List<String> types);
}
